/*
 * Copyright (C) 2018 CERN
 * Author: Maciej Suminski <maciej.suminski@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <QtCore>
#include <QObject>
#include <vector>
#include <memory>
#include <libmfipp.h>
#include <diot_crate.h>

class DiotController;
class QDialog;
class QGroupBox;
class QPushButton;

class DiotSlot : public QObject {
    Q_OBJECT

public:
    DiotSlot(QDialog* parent, DiotController& controller, int num, int idx);
    virtual ~DiotSlot() {}

    unsigned int GetInput() const {
        return vec_to_int(inputs);
    }

    unsigned int GetOutput() const {
        return vec_to_int(outputs);
    }

public slots:
    void SetInput(unsigned int state);
    void SetOutput(unsigned int state);

signals:
    void InputChanged(int slot, unsigned int state);
    void OutputChanged(int slot, unsigned int state);

protected:
    void action_click(int slot, int io);
    void update_inputs();
    void update_outputs(int slot);

    static unsigned int vec_to_int(const std::vector<bool>& vec);
    static bool int_to_vec(unsigned int integer, std::vector<bool>& vec);

    DiotController& controller;
    std::vector<bool> inputs, outputs;
    std::vector<QPushButton*> btn_state, btn_test;
    QGroupBox* groupBox;
    const int slot_nr;
    const int io_count;
};


class DiotController : public QObject {
    Q_OBJECT

public:
    DiotController(QDialog* dialog, int address, int slots_nr);
    void Start();

public slots:
    void UpdateInput(int slot, unsigned int state);
    void SetOutput(int slot, unsigned int state);

signals:
    void InputChanged(int slot, unsigned int state);

private:
    std::vector<std::unique_ptr<DiotSlot>> io_slots;
    std::unique_ptr<MasterFIP> mfip;
    std::unique_ptr<CycleSimple> cycle;
    std::unique_ptr<DIOT_Crate> crate;
    mstrfip_hw_cfg hw_cfg;
    mstrfip_sw_cfg sw_cfg;
};

#endif /* CONTROLLER_H */
