/*
 * Copyright (C) 2018 CERN
 * Author: Maciej Suminski <maciej.suminski@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "diot_crate.h"

void DIOT_Crate::SetOutput(int slot, int idx, bool state)
{
    if (slot > card_count) {
        throw std::runtime_error("DIOT_Crate::GetIO(): Too high slot number");
    }

    const int var_offset = getVarOffset(slot, idx);
    const uint8_t io_mask = getIOMask(idx);
    uint8_t data = GetProdVarData(var_offset);

    if((data & io_mask) == state)
        return;     // no update needed

    if (state)
        data |= io_mask;
    else
        data &= ~io_mask;

    SetProdVarData(var_offset, data);
}


bool DIOT_Crate::GetInput(int slot, int idx)
{
    if (slot > card_count) {
        throw std::runtime_error("DIOT_Crate::GetIO(): Too high slot number");
    }

    const uint8_t data = GetConsVarData(getVarOffset(slot, idx));
    return data & getIOMask(idx);
}


void DIOT_Crate::SetSlotOutputs(int slot, uint32_t state)
{
    const int var_offset = getVarOffset(slot, 0);

    {   // mutex locked scope
    auto lock = getProdLock();
    setProdDataLock(var_offset, (uint8_t)(state & 0xff), lock);
    setProdDataLock(var_offset+1, (uint8_t)(state >> 8 & 0xff), lock);
    setProdDataLock(var_offset+2, (uint8_t)(state >> 16 & 0xff), lock);
    setProdDataLock(var_offset+3, (uint8_t)(state >> 24 & 0xff), lock);
    }
}


uint32_t DIOT_Crate::GetSlotInputs(int slot)
{
    const int var_offset = getVarOffset(slot, 0);
    uint32_t ret = 0;

    {   // mutex locked scope
    auto lock = getConsLock();

    for(int i = 3; i >= 0; --i) {
        ret <<= 8;
        ret |= getConsDataLock(var_offset + i, lock);
    }
    }

    return ret;
}
